import { Component, OnInit } from '@angular/core';
import { HttpClient } from '@angular/common/http'
import { Router, ActivatedRoute } from '@angular/router'
import { CacheTokenService } from '../../service/cache-token.service'//引入服务
import { AbstractControl, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { NzMessageService } from 'ng-zorro-antd/message';
import { Base64 } from 'js-base64';
import { NzNotificationService } from 'ng-zorro-antd/notification';
import { NzSafeAny } from 'ng-zorro-antd/core/types';

declare var abp: any
@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.scss'],

})
export class LoginComponent implements OnInit {

  validateForm!: FormGroup;
  constructor(private http: HttpClient, private router: Router, private route: ActivatedRoute, public token: CacheTokenService, private fb: FormBuilder, private message: NzMessageService, private notificationService: NzNotificationService) {
    const { password } = MyValidators;
    this.validateForm = this.fb.group({
      userName: [null, [Validators.required]],
      password: [null, [Validators.required, password]],
      remember: [null],
      autologin: [null],
    });
  }

  ngOnInit() {

    //如果本地储存了账号密码信息，那就回显在输入框
    let account = localStorage.getItem('account')
    let password = localStorage.getItem('password')
    if (account && password) {
      this.validateForm.patchValue({
        userName: Base64.decode(localStorage.getItem('account')),
        password: Base64.decode(localStorage.getItem('password'))
      });
      this.remember = true //这一步是回显后让勾选框为选中状态
    }
    //自动登录
    if (localStorage.getItem('isautologin') == 'true') {
      this.submitForm()
      this.autologin = true //这一步是回显后让勾选框为选中状态
    }

  }

  errmsg: string = ''; //错误信息

  //记住密码
  rememberInfo() {
    // 判断用户是否勾选记住密码，如果勾选，在本地储存中储存登录信息
    if (this.remember) {
      localStorage.setItem("account", Base64.encode(this.validateForm.value.userName))
      localStorage.setItem("password", Base64.encode(this.validateForm.value.password))
    }
  }
  //自动登录
  autoLogin() {
    if (this.autologin) {
      localStorage.setItem("isautologin", 'true')
    }
  }

  remember: any//记住密码
  autologin: any//自动登录
  isLoading = false;
  messages
  encryptedAccessToken
  submitForm(): void {
    if (!this.remember) {
      localStorage.removeItem("account")
      localStorage.removeItem("password")
    }
    if (!this.autologin) {
      localStorage.removeItem("isautologin")
    }


    for (const i in this.validateForm.controls) {
      this.validateForm.controls[i].markAsDirty();
      this.validateForm.controls[i].updateValueAndValidity();
    }
    if (!this.validateForm.valid) {
      this.message.create('error', `请输入账号密码`);
      return
    }
    this.isLoading = true;
    this.http.post('/api/TokenAuth/Authenticate', {
      userNameOrEmailAddress: this.validateForm.value.userName,
      password: this.validateForm.value.password
    }).subscribe(
      (data: any) => {
        sessionStorage.setItem("token", data.result.accessToken);
        sessionStorage.setItem("encryptedAccessToken", data.result.encryptedAccessToken);
        // console.log('token', data.result)
        this.http.get('/api/services/app/Session/GetCurrentLoginInformations').subscribe((data: any) => {
          console.log('GetCurrentLoginInformations', data.result)
          sessionStorage.setItem('userdata', JSON.stringify(data.result.user))
          sessionStorage.setItem('userdataOfgasstation', JSON.stringify(data.result.user))
          sessionStorage.setItem('isDefaultPassword', JSON.stringify(data.result.user.isDefaultPassword))
          sessionStorage.setItem('isPasswordExpired', JSON.stringify(data.result.user.isPasswordExpired))
          this.isLoading = false;
          //记住密码
          this.rememberInfo()
          //自动登录
          this.autoLogin()
          if (data.result.user.roles.find((item) => {
            return item.name == 'ViolationPositiveVerifier'
          })) {
            this.router.navigate(['/records'])
            this.message.create('success', `登录成功`);
          } else {
            this.message.create('warning', `该账号没有预警审核权限`);
          }
        }, err => {
          this.isLoading = false;
        })

        //调用服务中的function刷新token
        // this.token.startUp()
      },
      (err) => {
        this.isLoading = false;
        this.message.create('error', err.error.error.details);
      }
    )
  }

  forget() {
    this.message.create('warning', `请联系管理员`);
  }



}
export type MyErrorsOptions = { 'zh-cn': string; en: string } & Record<string, NzSafeAny>;
export type MyValidationErrors = Record<string, MyErrorsOptions>;
export class MyValidators extends Validators {
  static password(control: AbstractControl): MyValidationErrors | null {
    const value = control.value;

    if (isEmptyInputValue(value)) {
      return null;
    }

    return isPassword(value) ? null : { mobile: { 'zh-cn': `长度 12 位以上，包含①大写字母、②小写字母、③数字、④特殊字符四种中的三种组合`, en: `Password phone number is not valid` } };
  }

}
function isEmptyInputValue(value: NzSafeAny): boolean {
  return value == null || value.length === 0;
}
function isPassword(value: string): boolean {
  return typeof value === 'string' && /^(?![a-zA-Z]+$)(?![A-Z0-9]+$)(?![A-Z\W_!@#$%^&*`~()-+=]+$)(?![a-z0-9]+$)(?![a-z\W_!@#$%^&*`~()-+=]+$)(?![0-9\W_!@#$%^&*`~()-+=]+$)[a-zA-Z0-9\W_!@#$%^&*`~()-+=]{12,99}$/.test(value);
}

